@@ -109,7 +109,41 @@ exact behavior of the functions they're calling, and trial and error is required
 to make the system do what one wants.
 
 *** Solution
-This module contains functionality to make any arbitrary function broadcastable.
+This module contains functionality to make any arbitrary function broadcastable,
+in either C or Python.
+
+**** Broadcasting rules
+A detailed description of broadcasting rules is available in the numpy
+documentation: http://docs.scipy.org/doc/numpy/user/basics.broadcasting.html
+
+In short:
+
+- The most significant dimension in a numpy array is the LAST one, so the
+  prototype of an input argument must exactly match a given input's trailing
+  shape. So a prototype shape of (a,b,c) accepts an argument shape of (......,
+  a,b,c), with as many or as few leading dimensions as desired.
+- The extra leading dimensions must be compatible across all the inputs. This
+  means that each leading dimension must either
+  - equal to 1
+  - be missing (thus assumed to equal 1)
+  - equal to some positive integer >1, consistent across all arguments
+- The output is collected into an array that's sized as a superset of the
+  above-prototype shape of each argument
+
+More involved example: A function with input prototype ( (3,), ('n',3), ('n',),
+('m',) ) given inputs of shape
+
+    (1,5,    3)
+    (2,1,  8,3)
+    (        8)
+    (  5,    9)
+
+will return an output array of shape (2,5, ...), where ... is the shape of each
+output slice. Note again that the prototype dictates the TRAILING shape of the
+inputs.
+
+**** Broadcasting in python
+
 This is invoked as a decorator, applied to the arbitrary user function. An
 example:
 
@@ -140,34 +174,6 @@ vectors of length 'n' each (same 'n' for the two inputs). This new
     >>> inner_product(a,b)
     array([ 305, 1250])
 
-A detailed description of broadcasting rules is available in the numpy
-documentation: http://docs.scipy.org/doc/numpy/user/basics.broadcasting.html
-
-In short:
-
-- The most significant dimension in a numpy array is the LAST one, so the
-  prototype of an input argument must exactly match a given input's trailing
-  shape. So a prototype shape of (a,b,c) accepts an argument shape of (......,
-  a,b,c), with as many or as few leading dimensions as desired.
-- The extra leading dimensions must be compatible across all the inputs. This
-  means that each leading dimension must either
-  - equal to 1
-  - be missing (thus assumed to equal 1)
-  - equal to some positive integer >1, consistent across all arguments
-- The output is collected into an array that's sized as a superset of the
-  above-prototype shape of each argument
-
-More involved example: A function with input prototype ( (3,), ('n',3), ('n',),
-('m',) ) given inputs of shape
-
-    (1,5,    3)
-    (2,1,  8,3)
-    (        8)
-    (  5,    9)
-
-will return an output array of shape (2,5, ...), where ... is the shape of each
-output slice. Note again that the prototype dictates the TRAILING shape of the
-inputs.
 
 Another related function in this module broadcast_generate(). It's similar to
 broadcast_define(), but instead of adding broadcasting-awareness to an existing
@@ -178,24 +184,35 @@ Stock numpy has some rudimentary support for all this with its vectorize()
 function, but it assumes only scalar inputs and outputs, which severaly limits
 its usefulness.
 
+**** Broadcasting in C
+
+A C-level flavor of broadcast_define() is available. It wraps C code in C loops.
+This is an analogue of PDL::PP (http://pdl.perl.org/PDLdocs/PP.html). Here the
+numpysane_pywrap module is used to produce C code that is compiled and linked
+into a python extension module. This takes more effort than python-level
+broadcasting, but the results have much less overhead, and run much faster.
+Please see the sample
+(https://github.com/dkogan/numpysane/blob/master/pywrap-sample).
+
+This is relatively new, so please let me know if you try it, and stuff does or
+does not work.
+
 *** New planned functionality
 
-In addition to this basic broadcasting support, I'm planning the following:
+The C broadcasting is functional, but a few more features are on the roadmap:
 
-- A C-level broadcast_define(). This would be the analogue of PDL::PP
-  (http://pdl.perl.org/PDLdocs/PP.html). This flavor of broadcast_define() would
-  be invoked by the build system to wrap C functions. It would implement
-  broadcasting awareness in C code it generates, which should work more
-  effectively for performance-sensitive inner loops. Currently broadcasting
-  loops are all implemented in python, and this can get noticeably slow for
-  large broadcasts.
+- It should be possible for some inputs/output to contain different data types
 
-- Automatic parallelization for broadcasted slices. Since each broadcasting loop
-  is independent, this is a very natural place to add parallelism.
+- And sometimes one would want to produce more than one output array for each
+  call, possibly with different types
 
-- Dimensions should support a symbolic declaration. For instance, one could want
-  a function to accept an input of shape (n) and another of shape (n*n). There's
-  no way to declare this currently, but there should be.
+- The prototype specification is not flexible enough. Maybe there's some
+  relationship between named dimensions that is known. If so, this should be
+  specify-able
+
+- Parallelization for broadcasted slices. Since each broadcasting loop is
+  independent, this is a very natural place to add parallelism. This is fairly
+  simple with OpenMP.
 
 ** Strangeness in core routines
 *** Problem
@@ -522,6 +539,10 @@ Broadcast-aware outer product.
 **** norm2
 Broadcast-aware 2-norm. norm2(x) is identical to inner(x,x)
 
+**** mag
+Broadcast-aware vector magnitude. mag(x) is functionally identical to
+sqrt(inner(x,x))
+
 **** trace
 Broadcast-aware trace.
 
@@ -535,17 +556,13 @@ The functions listed above are a start, but more will be added with time.
 
 import numpy as np
 from functools import reduce
+import itertools
 import types
 import inspect
 from distutils.version import StrictVersion
 
 # setup.py assumes the version is a simple string in '' quotes
-__version__ = '0.17'
-
-# object needed for fancy slices. m[:] is exactly the same as m[_colon], but
-# '_colon' is a normal python object, so it can be manipulated in ways that ':'
-# can't
-_colon = slice(None, None, None)
+__version__ = '0.21'
 
 def _product(l):
     r'''Returns product of all values in the given list'''
@@ -585,23 +602,43 @@ class NumpysaneError(Exception):
     def __str__(self):       return self.err
 
 
-def _eval_broadcast_dims( args, prototype, dims_extra, dims_named ):
+def _eval_broadcast_dims( args, prototype ):
     r'''Helper function to evaluate a given list of arguments in respect to a given
     broadcasting prototype. This function will flag any errors in the
-    dimensionality of the inputs. If no errors are detected, the outer shape of
-    the broadcast is stored in the dims_extra list, and the values of the
-    symbolic dimensions are stored in the dims_named dict
+    dimensionality of the inputs. If no errors are detected, it returns
+
+      dims_extra,dims_named
+
+    where
+
+      dims_extra is the outer shape of the broadcast
+        This is a list: the union of all the leading shapes of all the
+        arguments, after the trailing shapes of the prototype have been stripped
+
+      dims_named is the sizes of the named dimensions
+        This is a dict mapping dimension names to their sizes
 
     '''
 
+    # First I initialize dims_extra: the array containing the broadcasted
+    # slices. Each argument calls for some number of extra dimensions, and the
+    # overall array is as large as the biggest one of those
+    Ndims_extra = 0
+    for i_arg in range(len(args)):
+        Ndims_extra_here = len(args[i_arg].shape) - len(prototype[i_arg])
+        if Ndims_extra_here > Ndims_extra:
+            Ndims_extra = Ndims_extra_here
+    dims_extra = [1] * Ndims_extra
+
+
     def parse_dim( name_arg,
-                   shape_prototype, shape_arg, dims_extra, dims_named ):
+                   shape_prototype, shape_arg, dims_named ):
 
         def range_rev(n):
             r'''Returns a range from -1 to -n.
 
             Useful to index variable-sized lists while aligning their ends.'''
-            return [-i-1 for i in range(n)]
+            return range(-1, -n-1, -1)
 
 
 
@@ -612,20 +649,25 @@ def _eval_broadcast_dims( args, prototype, dims_extra, dims_named ):
             ndims_missing_here = len(shape_prototype) - len(shape_arg)
             shape_arg = (1,) * ndims_missing_here + shape_arg
 
+        # MAKE SURE THE PROTOTYPE DIMENSIONS MATCH (the trailing dimensions)
+        #
         # Loop through the dimensions. Set the dimensionality of any new named
         # argument to whatever the current argument has. Any already-known
         # argument must match
         for i_dim in range_rev(len(shape_prototype)):
-            if not isinstance(shape_prototype[i_dim], int) and \
-               shape_prototype[i_dim] not in dims_named:
-                dims_named[shape_prototype[i_dim]] = shape_arg[i_dim]
+
+            dim_prototype = shape_prototype[i_dim]
+
+            if not isinstance(dim_prototype, int):
+                # This is a named dimension. These can have any value, but ALL
+                # dimensions of the same name must thave the SAME value
+                # EVERYWHERE
+                if dim_prototype not in dims_named:
+                    dims_named[dim_prototype] = shape_arg[i_dim]
+                dim_prototype = dims_named[dim_prototype]
 
             # The prototype dimension (named or otherwise) now has a numeric
             # value. Make sure it matches what I have
-            dim_prototype = dims_named[shape_prototype[i_dim]] \
-                            if not isinstance(shape_prototype[i_dim], int) \
-                               else shape_prototype[i_dim]
-
             if dim_prototype != shape_arg[i_dim]:
                 raise NumpysaneError("Argument {} dimension '{}': expected {} but got {}".
                     format(name_arg,
@@ -636,13 +678,13 @@ def _eval_broadcast_dims( args, prototype, dims_extra, dims_named ):
         # I now know that this argument matches the prototype. I look at the
         # extra dimensions to broadcast, and make sure they match with the
         # dimensions I saw previously
-        ndims_extra_here = len(shape_arg) - len(shape_prototype)
+        Ndims_extra_here = len(shape_arg) - len(shape_prototype)
 
-        # This argument has ndims_extra_here dimensions to broadcast. The
+        # MAKE SURE THE BROADCASTED DIMENSIONS MATCH (the leading dimensions)
+        #
+        # This argument has Ndims_extra_here dimensions to broadcast. The
         # current shape to broadcast must be at least as large, and must match
-        if ndims_extra_here > len(dims_extra):
-            dims_extra[:0] = [1] * (ndims_extra_here - len(dims_extra))
-        for i_dim in range_rev(ndims_extra_here):
+        for i_dim in range_rev(Ndims_extra_here):
             dim_arg = shape_arg[i_dim - len(shape_prototype)]
             if dim_arg != 1:
                 if dims_extra[i_dim] == 1:
@@ -656,77 +698,44 @@ def _eval_broadcast_dims( args, prototype, dims_extra, dims_named ):
                                dim_arg))
 
 
-    # clear out the accumulators
-    dims_extra[:] = []
-    dims_named.clear()
+    dims_named = {} # parse_dim() adds to this
     for i_arg in range(len(args)):
         parse_dim( i_arg,
                    prototype[i_arg], args[i_arg].shape,
-                   dims_extra, dims_named )
+                   dims_named )
+
+    return dims_extra,dims_named
 
 
 
-def _broadcast_iter_dim( i_dims_extra,
-
-                         idx_slices, args, prototype,
-                         dims_extra ):
-    r'''Recursive generator to iterate through all the broadcasting slices.
-
-    Each recursive call loops through a single dimension. I can do
-    some of this with itertools.product(), and maybe using that
-    would be a better choice.
-
-    i_dims_extra is an integer indexing the current extra dimension
-    we're looking at.
-
-    idx_slices is an array of indices for each argument that is
-    filled in by this function. This may vary for each argument
-    because of varying prototypes and varying broadcasting shapes.
-
+def _broadcast_iter_dim( args, prototype, dims_extra ):
+    r'''Generator to iterate through all the broadcasting slices.
     '''
 
-    if idx_slices is None:
-        idx_slices = [[0]*(x.ndim-len(p)) + [_colon]*len(p) for p,x in zip(prototype,args)]
+    # pad the dimension of each arg with ones. This lets me use the full
+    # dims_extra index on each argument, without worrying about overflow
+    args = [ atleast_dims(args[i], -(len(prototype[i])+len(dims_extra)) ) for i in range(len(args)) ]
 
-    if i_dims_extra < len(dims_extra):
-        # more dimensions remaining. recurse
+    # per-arg dims_extra indexing varies: len-1 dimensions always index at 0. I
+    # make a mask that I apply each time
+    idx_slice_mask = np.ones( (len(args), len(dims_extra)), dtype=int)
+    for i in range(len(args)):
+        idx_slice_mask[i, np.array(args[i].shape,dtype=int)[:len(dims_extra)]==1] = 0
 
-        # idx_slices contains the indices for each argument.
-        # Two notes:
+    for idx_slice in itertools.product( *(range(x) for x in dims_extra) ):
+        # tuple(idx) because of wonky behavior differences:
+        #     >>> a
+        #     array([[0, 1, 2],
+        #            [3, 4, 5]])
         #
-        # 1. Any indices indexing above the first dimension should be
-        #    omitted
-        # 2. Indices into a higher dimension of length 1 should be left at 0
-        for dim in range(dims_extra[i_dims_extra]):
-            for x,p,idx_slice in zip(args,prototype,idx_slices):
-                x_dim = x.ndim - (len(p) + len(dims_extra) - i_dims_extra)
-                if x_dim >= 0 and x.shape[x_dim] > 1:
-                    idx_slice[x_dim] = dim
-
-            for x in _broadcast_iter_dim( i_dims_extra+1,
-
-                                          idx_slices, args, prototype,
-                                          dims_extra):
-                yield x
-        return
-
-    # This is the last dimension. Evaluate this slice.
-
-    # the "if idx else x" business if for 0-dimensional arrays.
-    # tuple(idx) because of wonky behavior differences:
-    #     >>> a
-    #     array([[0, 1, 2],
-    #            [3, 4, 5]])
-    #
-    #     >>> a[tuple((1,1))]
-    #     4
-    #
-    #     >>> a[list((1,1))]
-    #     array([[3, 4, 5],
-    #            [3, 4, 5]])
-    yield tuple( x[tuple(idx)] if idx else x for idx,x in zip(idx_slices, args) )
-
-
+        #     >>> a[tuple((1,1))]
+        #     4
+        #
+        #     >>> a[list((1,1))]
+        #     array([[3, 4, 5],
+        #            [3, 4, 5]])
+        yield tuple( args[i][tuple(idx_slice *
+                                   idx_slice_mask[i])] for i in range(len(args)) )
 
 
 def broadcast_define(prototype, prototype_output=None, out_kwarg=None):
@@ -970,9 +979,10 @@ def broadcast_define(prototype, prototype_output=None, out_kwarg=None):
             # make sure all the arguments are numpy arrays
             args = tuple(np.asarray(arg) for arg in args)
 
-            dims_extra = [] # extra dimensions to broadcast through
-            dims_named = {} # values of the named dimensions
-            _eval_broadcast_dims( args, prototype, dims_extra, dims_named )
+            # dims_extra: extra dimensions to broadcast through
+            # dims_named: values of the named dimensions
+            dims_extra,dims_named = \
+                _eval_broadcast_dims( args, prototype)
 
             # if no broadcasting involved, just call the function
             if not dims_extra:
@@ -984,38 +994,33 @@ def broadcast_define(prototype, prototype_output=None, out_kwarg=None):
             # output, and gather the results
             output = None
 
-            # reshaped output. I write to this array
-            output_flattened = None
-
             # substitute named variable values into the output prototype
-            prototype_output_local = None
+            prototype_output_expanded = None
             if prototype_output is not None:
-                prototype_output_local = [d if type(d) is int
+                prototype_output_expanded = [d if type(d) is int
                                           else dims_named[d] for d in prototype_output]
 
             # if the output was supposed to go to a particular place, set that
-            kwargs_dtype = {}
-            if 'dtype' in kwargs:
-                kwargs_dtype['dtype'] = kwargs['dtype']
             if out_kwarg and out_kwarg in kwargs:
                 output = kwargs[out_kwarg]
-                if prototype_output_local is not None:
-                    expected_shape = dims_extra + prototype_output_local
+                if prototype_output_expanded is not None:
+                    expected_shape = dims_extra + prototype_output_expanded
                     if output.shape != tuple(expected_shape):
                         raise NumpysaneError("Inconsistent output shape: expected {}, but got {}".format(expected_shape, output.shape))
             # if we know enough to allocate the output, do that
-            elif prototype_output_local is not None:
-                output = np.empty(dims_extra + prototype_output_local,
+            elif prototype_output_expanded is not None:
+                kwargs_dtype = {}
+                if 'dtype' in kwargs:
+                    kwargs_dtype['dtype'] = kwargs['dtype']
+                output = np.empty(dims_extra + prototype_output_expanded,
                                   **kwargs_dtype)
 
+            # reshaped output. I write to this array
             if output is not None:
-                output_flattened = output.reshape( (_product(dims_extra),) +
-                                                   output.shape[len(dims_extra):] )
+                output_flattened = clump(output, n=len(dims_extra))
 
             i_slice = 0
-            for x in _broadcast_iter_dim( 0,
-                                          None, args, prototype,
-                                          dims_extra ):
+            for x in _broadcast_iter_dim( args, prototype, dims_extra ):
 
                 # if the function knows how to write directly to an array,
                 # request that
@@ -1025,6 +1030,9 @@ def broadcast_define(prototype, prototype_output=None, out_kwarg=None):
                 sliced_args = x + args_passthru
                 result = func( *sliced_args, **kwargs )
 
+                if not isinstance(result, np.ndarray):
+                    result = np.array(result)
+
                 if output is None:
                     output = np.empty( dims_extra + list(result.shape),
                                        dtype = result.dtype)
@@ -1034,10 +1042,10 @@ def broadcast_define(prototype, prototype_output=None, out_kwarg=None):
                 elif not out_kwarg:
                     output_flattened[i_slice, ...] = result
 
-                if prototype_output_local is None:
-                    prototype_output_local = result.shape
-                elif result.shape != tuple(prototype_output_local):
-                    raise NumpysaneError("Inconsistent slice output shape: expected {}, but got {}".format(prototype_output_local, result.shape))
+                if prototype_output_expanded is None:
+                    prototype_output_expanded = result.shape
+                elif result.shape != tuple(prototype_output_expanded):
+                    raise NumpysaneError("Inconsistent slice output shape: expected {}, but got {}".format(prototype_output_expanded, result.shape))
 
                 i_slice = i_slice+1
 
@@ -1100,15 +1108,15 @@ def broadcast_generate(prototype, args):
     # make sure all the arguments are numpy arrays
     args = tuple(np.asarray(arg) for arg in args)
 
-    dims_extra = [] # extra dimensions to broadcast through
-    _eval_broadcast_dims( args, prototype, dims_extra, {} )
+    # dims_extra: extra dimensions to broadcast through
+    # dims_named: values of the named dimensions
+    dims_extra,dims_named = \
+        _eval_broadcast_dims( args, prototype )
 
     # I checked all the dimensions and aligned everything. I have my
     # to-broadcast dimension counts. Iterate through all the broadcasting
     # output, and gather the results
-    for x in _broadcast_iter_dim( 0,
-                                  None, args, prototype,
-                                  dims_extra ):
+    for x in _broadcast_iter_dim( args, prototype, dims_extra ):
         yield x
 
 
@@ -1529,7 +1537,7 @@ def atleast_dims(x, *dims):
     if isinstance(dims, list):
         dims[:] = [ d+num_new_axes if d >= 0 else d for d in dims ]
 
-    return x[ (np.newaxis,)*(num_new_axes) + (_colon,)*x.ndim ]
+    return x[ (np.newaxis,)*(num_new_axes) ]
 
 def mv(x, axis_from, axis_to):
     r'''Moves a given axis to a new position. Similar to numpy.moveaxis().
@@ -1708,9 +1716,9 @@ def reorder(x, *dims):
 # Note that this explicitly isn't done with @broadcast_define. Instead I
 # implement the internals with core numpy routines. The advantage is that these
 # are some of very few numpy functions that support broadcasting, and they do so
-# on the C level, so their broadcasting loop is FAST. Much more so than my
-# current @broadcast_define loop
-def dot(a, b, out=None):
+# in C, so their broadcasting loop is FAST. Much more so than my current
+# @broadcast_define loop
+def dot(a, b, out=None, dtype=None):
     r'''Non-conjugating dot product of two 1-dimensional n-long vectors.
 
     Synopsis:
@@ -1735,11 +1743,12 @@ def dot(a, b, out=None):
     function has no special handling: normal broadcasting rules are applied.
 
     '''
-    v = np.sum(a*b, axis=-1)
+    if out is not None and dtype is not None and out.dtype != dtype:
+        raise NumpysaneError("'out' and 'dtype' given explicitly, but the dtypes are mismatched!")
+
+    v = np.sum(a*b, axis=-1, out=out, dtype=dtype )
     if out is None:
         return v
-
-    out.setfield(v, out.dtype)
     return out
 
 # nps.inner and nps.dot are equivalent. Set the functionality and update the
@@ -1759,7 +1768,7 @@ inner.__doc__ = doc
 # are some of very few numpy functions that support broadcasting, and they do so
 # on the C level, so their broadcasting loop is FAST. Much more so than my
 # current @broadcast_define loop
-def vdot(a, b, out=None):
+def vdot(a, b, out=None, dtype=None):
     r'''Conjugating dot product of two 1-dimensional n-long vectors.
 
     vdot(a,b) is equivalent to dot(np.conj(a), b)
@@ -1789,7 +1798,7 @@ def vdot(a, b, out=None):
     broadcasting rules are applied.
 
     '''
-    return dot(np.conj(a), b, out=out)
+    return dot(np.conj(a), b, out=out, dtype=dtype)
 
 @broadcast_define( (('n',), ('n',)), prototype_output=('n','n'), out_kwarg='out' )
 def outer(a, b, out=None):
@@ -1822,8 +1831,8 @@ def outer(a, b, out=None):
 # Note that this explicitly isn't done with @broadcast_define. Instead I
 # implement the internals with core numpy routines. The advantage is that these
 # are some of very few numpy functions that support broadcasting, and they do so
-# on the C level, so their broadcasting loop is FAST. Much more so than my
-# current @broadcast_define loop
+# in C, so their broadcasting loop is FAST. Much more so than my current
+# @broadcast_define loop
 def norm2(a, **kwargs):
     r'''Broadcast-aware 2-norm. norm2(x) is identical to inner(x,x)
 
@@ -1844,6 +1853,42 @@ def norm2(a, **kwargs):
     '''
     return inner(a,a, **kwargs)
 
+def mag(a, out=None):
+    r'''Magnitude of a vector. mag(x) is functionally identical to sqrt(inner(x,x))
+
+    Synopsis:
+
+        >>> import numpy as np
+        >>> import numpysane as nps
+
+        >>> a = np.arange(3)
+        >>> a
+        array([0, 1, 2])
+
+        >>> nps.mag(a)
+        2.23606797749979
+
+    This is a convenience function to compute a magnitude of a vector, with full
+    broadcasting support. If and explicit "out" array isn't given, we produce
+    output of dtype=float. Otherwise "out" retains its dtype
+
+    '''
+
+    if out is None:
+        out = inner(a,a, dtype=float)
+
+        if not isinstance(out, np.ndarray):
+            # given two vectors, and without and 'out' array, inner() produces a
+            # scalar, not an array. So I can't updated it inplace, and just
+            # return a copy
+            return np.sqrt(out)
+    else:
+        inner(a,a, out=out)
+
+    # in-place sqrt
+    np.sqrt.at(out,())
+    return out
+
 @broadcast_define( (('n','n',),), prototype_output=() )
 def trace(a):
     r'''Broadcast-aware trace
@@ -1942,19 +1987,10 @@ def matmult2(a, b, out=None):
             return out
         return a*b
 
-    prepended = False
-    if len(a.shape) == 1:
-        a = a[np.newaxis, :]
-        if out is not None:
-            out = out[np.newaxis, ...]
-        prepended = True
-
     if len(b.shape) == 1:
         b = b[np.newaxis, :]
 
     o = np.matmul(a,b, out)
-    if prepended:
-        o.resize( o[...,0,:].shape )
     return o
 
 def matmult( *args ):
@@ -1990,7 +2026,21 @@ def matmult( *args ):
 
     This multiplies N matrices together by repeatedly calling matmult2() for
     each adjacent pair. Unlike matmult2(), the arguments MUST all be matrices to
-    multiply, an 'out' kwarg for the output is not supported here.
+    multiply. The 'out' kwarg for the output is not supported here.
+
+    This function supports broadcasting fully, in C internally
 
     '''
     return reduce( matmult2, args )
+
+
+
+# I use np.matmul at one point. This was added in numpy 1.10.0, but
+# apparently I want to support even older releases. I thus provide a
+# compatibility function in that case. This is slower (python loop instead of C
+# loop), but at least it works
+if not hasattr(np, 'matmul'):
+    @broadcast_define( (('n','m'),('m','o')), ('n','o'))
+    def matmul(a,b, out=None):
+        return np.dot(a,b,out)
+    np.matmul = matmul
